/***************************************************************************
    file:           general.cpp
    description:    general use
    begin:          09/06/2024
    Copyright:      Giuseppe Della Bianca<giusdbg@gmail.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <cmath>

#include <qstring.h>
#include <qstandardpaths.h>

#include "general.h"


Q_LOGGING_CATEGORY(qlcMain, "ks5.Main")
Q_LOGGING_CATEGORY(qlcLMSensorsAlarms, "ks5.LMSensorsAlarms")

// #define Icon(x) KGlobal::instance()->iconLoader()->loadIcon( x, KIcon::NoGroup, KIcon::SizeMedium )
QString findIcon( const char *name )
{
    QString qsName;

    qsName = qsFL1 ( "icons/" ) + qsFL1( name );

    return QStandardPaths::locate( QStandardPaths::AppDataLocation, qsName );
}

/***************************************************************************
    double compare
 ***************************************************************************/

int doubleCompare( double rif, double value )
{
    int result;

    double diff;

    result = 0;


    diff = rif - value;

    if ( !qFuzzyIsNull ( diff ) )
        result = diff > doubleEPS ? 1 : -1;

    return result;
}

/***************************************************************************
    qreal compare
 ***************************************************************************/

int qrealCompare( qreal rif, qreal value )
{
    int result;

    qreal diff;

    result = 0;


    diff = rif - value;

    if ( !qFuzzyIsNull ( diff ) )
        result = diff > qrealEPS ? 1 : -1;

    return result;
}

/***************************************************************************
    qreal division
 ***************************************************************************/

qreal qrealDivision( qreal dividend, qreal divisor )
{
    qreal result;

    result = 0;


    if ( !qFuzzyIsNull ( divisor ) )
        result = dividend / divisor;

    return result;
}

/***************************************************************************
    calculate rounding for boundary values
 ***************************************************************************/

qreal roundBoundValue ( qreal value, bool enabMax )
{
    qreal absValue, stepValue, roundValue;

    absValue = abs( value );

    roundValue = value;

    stepValue = 1.0;

    if ( qrealCompare ( absValue, 1000.0 ) > 0 )
        stepValue = 100.0;
    else if ( qrealCompare ( absValue, 100.0 ) > 0 )
        stepValue = 10.0;
    else if ( qrealCompare ( absValue, 10.0 ) > 0 )
        stepValue = 5.0;
    else if ( qrealCompare ( absValue, 1.0 ) > 0 )
        stepValue = 1.0;

    if ( enabMax )
        roundValue = ceil( value / stepValue ) * stepValue;
    else
        roundValue = floor( value / stepValue ) * stepValue;

    return roundValue;
}

/***************************************************************************
    calculate the maximum commm divisor
 ***************************************************************************/

void maxCommonDivisor ( QList<int> &qlISize, int &iMaxCommDiv, int &iMin, int &iMax )
{
    int iCurr;

    iMaxCommDiv = iMin = iMax = 0;

    if ( ! qlISize.empty() )
    {
        iMin = *std::min_element( qlISize.begin(), qlISize.end() );
        iMax = *std::max_element( qlISize.begin(), qlISize.end() );
    }

    for( iCurr = iMin; iCurr >= 1; iCurr-- )
    {
        if (std::all_of( qlISize.cbegin(), qlISize.cend(), [iCurr]( int intItem ){ return intItem % iCurr == 0; } ) )
        {
            iMaxCommDiv = iCurr;

            break;
        }
    }
}
