/***************************************************************************                          
    file:           generalcfgdesign.cpp
    description:    general configuration design
    begin:          09/06/2024
    Copyright:      Giuseppe Della Bianca<giusdbg@gmail.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qvariant.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qgroupbox.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qapplication.h>

#include <kaboutdata.h>
#include <klocalizedstring.h>

#include "general.h"
#include "generalcfgdesign.h"


/*
 *  Constructs a GeneralCfgDesign as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 */
GeneralCfgDesign::GeneralCfgDesign( KsensorsMain *ksensorsMainIn, QWidget *parent, const QString &name, Qt::WindowFlags fl ) : QWidget( parent /*, name */, fl )
{
    ksensorsMain = ksensorsMainIn;
    
    QGridLayout *qglAppear, *qglPref, *qglUpdTime;
    QVBoxLayout *qvblGaugeSize, *qvblChartSize;
    QHBoxLayout *qhblChartSize;
    
    if ( name.isEmpty() )
	setObjectName( "GeneralCfgDesign" );

    GeneralCfgDesignLayout = new QGridLayout( this ); 
    
    qtwMain = new QTabWidget( this );
    
    qwAppear = new QWidget( qtwMain );
    qglAppear = new QGridLayout( qwAppear );
    
    qwPref = new QWidget( qtwMain );
    qglPref = new QGridLayout( qwPref );
        
    qgbMeterAppear = new QGroupBox( qwAppear /*, "qgbMeterAppear" */ );
    qgbMeterAppear->setAlignment( Qt::Vertical );
//    qgbMeterAppear->layout()->setSpacing( 6 );
//    qgbMeterAppear->layout()->setMargin( 11 ); 
    qglMeterAppear = new QGridLayout( qgbMeterAppear );
    qglMeterAppear->setAlignment( Qt::AlignTop );


    qvblGaugeSize = new QVBoxLayout();
    qlGaugeSize = new QLabel( qgbMeterAppear );
    qcbGaugeSize = new QComboBox( qgbMeterAppear );
//    qcbGaugeSize->setGeometry( QRect(160, 50, 87, 32) );

    qvblChartSize = new QVBoxLayout();
    qhblChartSize = new QHBoxLayout();
    qlChartSize = new QLabel( qgbMeterAppear );
    qlChartSizeSep = new QLabel( qgbMeterAppear );
    qlChartSizeSep->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Preferred );
    qcbChartWidth = new QComboBox( qgbMeterAppear );
    qcbChartHeight = new QComboBox( qgbMeterAppear );

    addMeterSize();
    
    qvblGaugeSize->addWidget( qlGaugeSize );
    qvblGaugeSize->addWidget( qcbGaugeSize );

    qvblChartSize->addWidget( qlChartSize );
    qvblChartSize->addLayout( qhblChartSize );
    qhblChartSize->addWidget( qcbChartWidth );
    qhblChartSize->addWidget( qlChartSizeSep );
    qhblChartSize->addWidget( qcbChartHeight );

    qglMeterAppear->addLayout( qvblGaugeSize, 0, 0, 1, 1 );
    qglMeterAppear->addLayout( qvblChartSize, 0, 1, 1, 1 );

//    qglMeterAppear->addWidget ( qcbPanelSize, 0, 0 );    
    qglAppear->addWidget( qgbMeterAppear, 0, 0 );

    qcbPanelSolid = new QCheckBox ( qwAppear );
    qcbPanelSolid->setChecked( true );
    
    qglMeterAppear->addWidget( qcbPanelSolid, 1, 0 );
    
    qgbPanelOpacity = new QGroupBox( qwAppear );
    qgbPanelOpacity->setFlat( true );
    qhblPanelOpacity = new QHBoxLayout( qgbPanelOpacity );
    qhblPanelOpacity->setAlignment( Qt::AlignLeft );
    qhblPanelOpacity->setContentsMargins( 0, 0, 0, 0 );
    
    qlPanelOpacityLabel = new QLabel ( qwAppear );
    qlPanelOpacityLabel->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Preferred );
    qdsbPanelOpacityValue = new QDoubleSpinBox ( qwAppear );
    qdsbPanelOpacityValue->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Preferred );
    qdsbPanelOpacityValue->setDecimals ( 0 );
    qdsbPanelOpacityValue->setMinimum( 0.0 );
    qdsbPanelOpacityValue->setMaximum( 100.0 );
    qdsbPanelOpacityValue->setValue( 70 );
    qsiPanelOpacitySpacer = new QSpacerItem( 0, 0, QSizePolicy::Maximum, QSizePolicy::Preferred );
    
    qhblPanelOpacity->addWidget ( qlPanelOpacityLabel );
    qhblPanelOpacity->addWidget ( qdsbPanelOpacityValue );    
    qhblPanelOpacity->addItem( qsiPanelOpacitySpacer );
    qglMeterAppear->addWidget( qgbPanelOpacity, 1, 1 );

    qcbWindowsSolid = new QCheckBox ( qwAppear );
    qcbWindowsSolid->setChecked( true );

    qglMeterAppear->addWidget( qcbWindowsSolid, 2, 0 );
    
    qgbWindowsOpacity = new QGroupBox( qwAppear );
    qgbWindowsOpacity->setFlat( true );
    qhblWindowsOpacity = new QHBoxLayout( qgbWindowsOpacity );
    qhblWindowsOpacity->setContentsMargins( 0, 0, 0, 0 );
    
    qlWindowsOpacityLabel = new QLabel ( qwAppear );
    qlWindowsOpacityLabel->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Preferred );
    qdsbWindowsOpacityValue = new QDoubleSpinBox ( qwAppear );
    qdsbWindowsOpacityValue->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Preferred );
    qdsbWindowsOpacityValue->setDecimals ( 0 );
    qdsbWindowsOpacityValue->setMinimum( 0.0 );
    qdsbWindowsOpacityValue->setMaximum( 100.0 );
    qdsbWindowsOpacityValue->setValue( 70 );
    qsiWindowsOpacitySpacer = new QSpacerItem( 0, 0, QSizePolicy::Maximum, QSizePolicy::Preferred );
    
    qhblWindowsOpacity->addWidget ( qlWindowsOpacityLabel );
    qhblWindowsOpacity->addWidget ( qdsbWindowsOpacityValue );    
    qhblWindowsOpacity->addItem( qsiWindowsOpacitySpacer );
    qglMeterAppear->addWidget( qgbWindowsOpacity, 2, 1 );
    
    boxGeneralPalette = new QGroupBox( qwAppear );
    boxGeneralPalette->setMinimumSize( QSize( 0, 90 ) );
    qglAppear->addWidget( boxGeneralPalette, 1, 0 );
    
    GroupBox2 = new QGroupBox( qwAppear );
    GroupBox2->setAlignment( Qt::Vertical );
//    GroupBox2->layout()->setSpacing( 6 );
//    GroupBox2->layout()->setMargin( 11 );
    GroupBox2Layout = new QVBoxLayout( GroupBox2 );
    GroupBox2Layout->setAlignment( Qt::AlignTop );

    CheckBoxAutoStart = new QCheckBox( GroupBox2 /*, "CheckBoxAutoStart" */ );
    GroupBox2Layout->addWidget( CheckBoxAutoStart );

    qglAppear->addWidget( GroupBox2, 2, 0 );
    Spacer28 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );        
    qglAppear->addItem( Spacer28, 3, 0 );
    
    qtwMain->addTab( qwAppear, QString() );
        
    qgbUpdTime = new QGroupBox( qwPref );
    qgbUpdTime->setAlignment( Qt::Vertical );
    
    qglUpdTime = new QGridLayout( qgbUpdTime );
    qglUpdTime->setAlignment( Qt::AlignTop );

    qdsbUpdTime = new QDoubleSpinBox( qgbUpdTime );
    qdsbUpdTime->setSuffix ( qsFL1( "s" ) );
    qdsbUpdTime->setDecimals( 1 );
    qdsbUpdTime->setSingleStep( 0.1 );
    qdsbUpdTime->setMinimum( 0.5 );
    qdsbUpdTime->setValue( 5.0 );

    qglUpdTime->addWidget( qdsbUpdTime, 0, 0 );

    qsUpdTime = new QSlider( qgbUpdTime );
    qsUpdTime->setPageStep( 5 );
    qsUpdTime->setSingleStep( 5 );
    qsUpdTime->setMinimum( 5 );
    qsUpdTime->setMaximum( 600 );
    qsUpdTime->setValue( 50 );
    qsUpdTime->setOrientation( Qt::Horizontal );

    qglUpdTime->addWidget( qsUpdTime, 0, 1 );

    qglPref->addWidget( qgbUpdTime, 1, 0 );
    qtwMain->addTab( qwPref, QString() );

    GeneralCfgDesignLayout->addWidget( qtwMain, 0, 0 );
    
    languageChange();
//    resize( QSize(343, 239).expandedTo(minimumSizeHint()) );
//   clearWState( WState_Polished );
    
    connect( qcbPanelSolid, &QAbstractButton::toggled, this, &GeneralCfgDesign::slotPanelSolidToggled );
    connect( qcbWindowsSolid, &QAbstractButton::toggled, this, &GeneralCfgDesign::slotWindowsSolidToggled );    
    connect( qsUpdTime, SIGNAL( valueChanged( int) ), this, SLOT( slotUpdateTimeSliderValue( int ) ) );
    connect( qdsbUpdTime, SIGNAL( valueChanged( double ) ), this, SLOT( slotUpdateTimeSpinValue( double ) ) );
}

/*
 *  Destroys the object and frees any allocated resources
 */
GeneralCfgDesign::~GeneralCfgDesign()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void GeneralCfgDesign::languageChange()
{
    const KAboutData kadAboutData = KAboutData::applicationData();
    
    setWindowTitle( tr2i18n( "GeneralCfg" ) );
        
    qgbMeterAppear->setTitle( tr2i18n( "Panels appearance" ) );

    qlGaugeSize->setText ( qsFL1( "Gauges size" ) );
    qcbGaugeSize->setCurrentIndex( qcbGaugeSize->findData ( qsFL1( "64 x 64" ), ( Qt::UserRole + 1 ) ) );
    
    qlChartSize->setText ( qsFL1( "Chart size" ) );
    qlChartSizeSep->setText ( qsFL1( " x " ) );
    qcbChartWidth->setCurrentIndex( qcbChartWidth->findData ( qsFL1( "96" ), ( Qt::UserRole + 1 ) ) );
    qcbChartHeight->setCurrentIndex ( qcbChartHeight->findData ( qsFL1( "96" ), ( Qt::UserRole + 1 ) ) );

    qcbPanelSolid->setText( qsFL1( "Solid" ) );
    qlPanelOpacityLabel->setText( qsFL1( "Opacity" ) );
    qdsbPanelOpacityValue->setSuffix( qsFL1( "%" ) );

    qcbWindowsSolid->setText( qsFL1( "Windows Solid" ) );
    qlWindowsOpacityLabel->setText( qsFL1( "Windows Opacity" ) );
    qdsbWindowsOpacityValue->setSuffix( qsFL1( "%" ) );


    boxGeneralPalette->setTitle( tr2i18n( "Colors" ) );
    GroupBox2->setTitle( tr2i18n( "Startup" ) );
    CheckBoxAutoStart->setText( tr2i18n( ( qsFL1( "Autostart " ) + kadAboutData.displayName() + qsFL1( " on KDE startup" ) ).toLatin1().data() ) );
    
    qtwMain->setTabText( qtwMain->indexOf( qwAppear ), tr2i18n( "Appearance" ) );
    qtwMain->setTabText( qtwMain->indexOf( qwPref ), tr2i18n( "Preferences" ) );
    
    qgbUpdTime->setTitle( tr2i18n( "Update interval" ) );

}

/***************************************************************************
    add meter size
 ***************************************************************************/

void GeneralCfgDesign::addMeterSize()
{
    int itemNum;

    QString qsItem;

    QList<int>::iterator iQlInt;

    itemNum = -1;
    for ( iQlInt = ksensorsMain->qlIGaugeSizeList.begin(); iQlInt != ksensorsMain->qlIGaugeSizeList.end(); iQlInt++)
    {
        itemNum++;

        qsItem.clear();
        qsItem.append ( QString::number( *iQlInt ) ).append ( qsFL1( " x " ) ).append ( QString::number( *iQlInt ) );

        qcbGaugeSize->insertItem( itemNum, tr2i18n( qsItem.toLatin1().data() ) );
        qcbGaugeSize->setItemData( itemNum, QString::number( *iQlInt ), ( Qt::UserRole + 1 ) );
    }

    itemNum = -1;
    for ( iQlInt = ksensorsMain->qlIChartWidthList.begin(); iQlInt != ksensorsMain->qlIChartWidthList.end(); iQlInt++)
    {
        itemNum++;

        qsItem = QString::number( *iQlInt );

        qcbChartWidth->insertItem( itemNum, tr2i18n( qsItem.toLatin1().data() ) );
        qcbChartWidth->setItemData( itemNum, QString::number( *iQlInt ), ( Qt::UserRole + 1 ) );
    }

    itemNum = -1;
    for ( iQlInt = ksensorsMain->qlIChartHeightList.begin(); iQlInt != ksensorsMain->qlIChartHeightList.end(); iQlInt++)
    {
        itemNum++;

        qsItem = QString::number( *iQlInt );

        qcbChartHeight->insertItem( itemNum, tr2i18n( qsItem.toLatin1().data() ) );
        qcbChartHeight->setItemData( itemNum, QString::number( *iQlInt ), ( Qt::UserRole + 1 ) );
    }

} // void addMeterSize()

void GeneralCfgDesign::slotPanelSolidToggled( bool checked )
{
    if ( checked )
        qdsbPanelOpacityValue->setValue( 100.0 );
    else
        qdsbPanelOpacityValue->setValue( 70.0 );
}

void GeneralCfgDesign::slotWindowsSolidToggled( bool checked )
{
    if ( checked )
        qdsbWindowsOpacityValue->setValue( 100.0 );
    else
        qdsbWindowsOpacityValue->setValue( 70.0 );
}

void GeneralCfgDesign::slotUpdateTimeSliderValue( int value )
{
    qdsbUpdTime->setValue( ( value / 10.0 ) );        
}

void GeneralCfgDesign::slotUpdateTimeSpinValue( double value )
{
    qsUpdTime->setValue( ( value * 10.0 ) );        
}

//#include "generalcfgdesign.moc"
