/***************************************************************************
    file:           ksensorsmain.h
    description:    ksensors main
    begin:          09/06/2024
    Copyright:      Giuseppe Della Bianca<giusdbg@gmail.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _KSENSORSMAIN_H_
#define _KSENSORSMAIN_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qobject.h>
#include <qpointer.h>
#include <qmenu.h>
#include <qdialog.h>

#include "ksensorssetting.h"
#include "sensordevslist.h"
#include "sensorpanel.h"
#include "panelswindow.h"
#include "lmsensordockpanel.h"
#include "lmsensorsalarms.h"

#include "ksensorscfg.h"

class KSensorsCfg;
class LMSensorDockPanel;
class LMSensorsAlarms;
class PanelsWindow;

/**
  *@original author Miguel Novas
  */

class KsensorsMain : public QObject, public ksensorsSetting
{

	Q_OBJECT

public:
    enum Update 
    {
        Default = 0x00000000,
        Initialize = 0x00000001,
        GlobalTimer = 0x00000002,
        ReadConfig = 0x00000004,
        UpdateIntervalList = 0x00000008,
        UpdateSensorsList = 0x00000010
//                    = 0x00000020
//                    = 0x00000040
    };

    Q_DECLARE_FLAGS(Updates, Update)

    struct T_UpdateData
    {
        bool withChange;

        int genUpdateInterval;

        QString sensorName;
        QList<int> updateIntervalList;

        void init()
        {
            withChange = false;

            genUpdateInterval = 0;
            sensorName.clear();
            updateIntervalList.clear();
        }
    };

    KsensorsMain(QObject *parent = nullptr, const QString &name = QString() );
    ~KsensorsMain();

  void init();
  void createMenu ( QMenu *&qmMenu, QAction *&qaMinRest );
  
    void updateValue( const QString & name );

    int gaugeIncrWidth;
    int gaugeIncrHeight;
    int chartIncrWidth;
    int chartIncrHeight;

    QList<int> qlIGaugeSizeList;
    QList<int> qlIChartWidthList;
    QList<int> qlIChartHeightList;

    QPointer<SensorDevsList> sensorDevsList;
    QPointer<PanelsWindow> gaugePanelsWindow;
    QPointer<PanelsWindow> chartPanelsWindow;

public Q_SLOTS:
  void saveConfig ( bool enabSync );	

    void updateSensorsPanelsDocks( const QString &name,bool withChange );

protected Q_SLOTS:

  void mouseEventReceived(QMouseEvent *e);
  void minimizeRestoreWidget();
  void createConfigWidget();
  void updateMenu ( bool withGauge, bool withChart );
  void showAbout();
  void close();

private:

    int countSensors;
   
  QPointer<KSensorsCfg>     sensorsCfg;
  QPointer<LMSensorsAlarms> sensorsAlarm;

  QMenu *qmContext;
  QAction *qaMinimizeRestore;

    int idTimer;
    int updateInterval;
    int updateIntervalTimer;

    void updateConfig ( Updates updatesType, struct T_UpdateData &updateData );
    void updateSensorsConfig ( Updates updatesType, struct T_UpdateData &updateData );

  void createDockWidgets();  
  void createDockSensor(Sensor *sensor);
  void deleteDockSensor(LMSensorDockPanel *sensor);

    void createShowWidget( Updates updatesType );

  void createAlarmControl();

  void setMeterSizeList();

  void timerEvent( QTimerEvent * ) override;
};

Q_DECLARE_OPERATORS_FOR_FLAGS(KsensorsMain::Updates)

#endif // _KSENSORSMAIN_H
