/***************************************************************************
                          lmsensorschip.cpp  -  description
                             -------------------
    begin                : Wed Feb 20 2002
    Copyright	Giuseppe Della Bianca<giusdbg@gmail.com>

    original author	 : Miguel Novas
    email                : michaell@teleline.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include <sensors/sensors.h>

#include "general.h"
#include "lmsensorschip.h"


//****************************************************************************
// Public methods
//****************************************************************************

LMSensorsChip::LMSensorsChip( SensorDevsList *sensorDevsList, const sensors_chip_name *chip, const QString &name ) : SensorDev( sensorDevsList, name )
{
    chip_name = chip;
    
    setClass(Sensor::lmSensor);

    setObjectName( LMSensorsChip::getDevLongName( chip_name ) );
    
    setShortName ( LMSensorsChip::getDevShortName( chip_name ) );
    setLongName ( LMSensorsChip::getDevLongName( chip_name ) );
    setSubName ( LMSensorsChip::getDevSubName( chip_name ) );

    setDescription( LMSensorsChip::getDevDesc( chip_name ) );

 readConfig();
 createSensors();
}

LMSensorsChip::~LMSensorsChip()
{
}

/***************************************************************************
    get chip name, names e description of sensor device
 ***************************************************************************/

const sensors_chip_name *LMSensorsChip::getChipName()
{
    return chip_name;
}

/* warning static function */
QString LMSensorsChip::getDevShortName( const sensors_chip_name *chipname )
{
    QString devShortName;

    devShortName = qsFL1( chipname->prefix );

    return devShortName;
}

QString LMSensorsChip::getDevLongName ( const sensors_chip_name *chipname )
{
    QString devLongName;

    devLongName = LMSensorsChip::getDevDesc ( chipname );

    return devLongName;
}

QString LMSensorsChip::getDevSubName( const sensors_chip_name *chipname )
{
    QString devSubName;

    devSubName = qsFL1( chipname->path );

    return devSubName;
}

QString LMSensorsChip::getDevDesc( const sensors_chip_name *chipname )
{
    char buffer[256 + 1];

    QString devDesc;

    *buffer = '\0';
     if ( sensors_snprintf_chip_name(buffer, 256, chipname) < 0 )
        buffer[256] = '\0';

    devDesc = qsFL1( buffer );

    return devDesc;
}

void LMSensorsChip::createSensors()
{
 if(!chip_name) return;
 int nr1= 0;
 int nr2= 0;
#if SENSORS_API_VERSION < 0x400 /* libsensor 3 code */
 const sensors_feature_data *data;
 data= sensors_get_all_features(*chip_name, &nr1, &nr2);
#else
 const sensors_feature *data;
 data= sensors_get_features(chip_name, &nr1);
#endif
 while(data) {
   LMSensor *sensor= new LMSensor(this);
   if( !sensor->init(&data,&nr1,&nr2) )
     delete sensor;
 }
}

void LMSensorsChip::updateSensors ( bool enabUpdate )
{
 const QObjectList *sensors= &children();

 if(sensors)
 {
   for ( QObjectList::ConstIterator it = sensors->begin(); it != sensors->end(); ++it ) 
   {
     LMSensor *sensor = (LMSensor *)(*it);
 
   if ( sensor )
   {
      if(sensor->monitorized())
      {
// qDebug() << "\033[91m" << "qDebug(): " << QDateTime::currentDateTime() << " LMSensorsChip::updateSensors()" << "updateValue " << sensor->getLongName() << "\033[0m";

          sensor->updateValue ( enabUpdate );
      }
   }
   else
      break;
     
   }    
 }

 setWithGlobalTimerUpdate ( false );
}

