

#include <qapplication.h>
#include <qboxlayout.h>

#include <qmainwindow.h>
#include <qgraphicslayout.h>

// #include "sensorpanel.h"
#include "qlchart.h"

QLineChart::QLineChart ( QWidget *parent ): QWidget( parent )
{
    struct T_ParData parData;
    struct T_ValData valData;

    setParent( parent );

    qlsSeries1 = qlsSeries2 = qlsSeries3 = qlsSeries4 = nullptr;

    valChart.init();

    tickYAxis = 5;
    tickXAxis = 5;
    qrXAxisStep = 1.0;


    qcvView = new QChartView ( &qcChart, this );    
    qcvView->setDisabled( true );
//    qcvView->setParent( this );
    qcvView->setRenderHint( QPainter::Antialiasing );

    qcChart.addAxis( &qvaXAxis, Qt::AlignBottom );
    qcChart.addAxis( &qvaYAxis, Qt::AlignLeft );

    qlsSeries1 = new QLineSeries();
    qcChart.addSeries( qlsSeries1 );
    qpSeries = qlsSeries1->pen();
    qcChart.removeSeries( qlsSeries1 );
    qlsSeries1 = nullptr;

//    qlsSeries->append( 5, 1 );
/*  qlsSeries1->setName( qsFL1 ( "Line 1" ) );
    qlsSeries1->attachAxis( &qvaXAxis );
    qlsSeries1->attachAxis( &qvaYAxis ); */
//    QPen green(Qt::red);
//    green.setWidth(3);
//    qlsSeries->setPen(green);

//  qvaAxisY.setLabelsFont( qfLabel );
    qvaYAxis.setTickCount( tickYAxis + 1 );
    qvaYAxis.setLabelFormat( qsFL1( "%g" ) );
    qvaYAxis.setRange ( -5.0, 1.0 );

//  qvaAxisX.setLabelsFont( qfLabel );
    qvaXAxis.setTickCount( tickXAxis + 1 );
    qvaXAxis.setLabelFormat( qsFL1 ( "%g" ) );
    qvaXAxis.setRange( 0.0, 10.0 );
    qvaXAxis.setReverse ( true );


    qcChart.setAnimationOptions( QChart::NoAnimation );
//    qcChart.setTitle( qsFL1 ("Dynamic line") );
    qcChart.legend()->hide();
    qcChart.setMargins( QMargins( 0, 0, 0, 0 ) );
    qcChart.layout()->setContentsMargins( 0, 0, 0, 0 );
    qcChart.setBackgroundVisible( false );


    qlValue1 = new QLabel ( qcvView );
    qlValue2 = new QLabel ( qcvView );
    qlValue3 = new QLabel ( qcvView );
    qlValue4 = new QLabel ( qcvView );
    qlDescr1 = new QLabel ( qcvView );    
    qlDescr2 = new QLabel ( qcvView );
    
//    qlDescr2->setPalette( getPaletteColor( Qt::black ) );

    valChart.visibleValue1 = valChart.visibleValue2 = valChart.visibleValue3 = valChart.visibleValue4 = true;
    valChart.visibleDescr1 = valChart.visibleDescr2 = true;
    
    adjustSize();


//    updateDescr();
    parData.init ( true );

    parData.qvValue1 = parData.qvDescr1 = true;

    setValueVisible( parData );

    parData.qvValue1 = qsFL1( "val. 1" );
    parData.qvValue2 = qsFL1( "val. 2" );
    parData.qvValue3 = qsFL1( "val. 3" );
    parData.qvValue4 = qsFL1( "val. 4" );
    parData.qvDescr1 = qsFL1( "descr. 1" );
    parData.qvDescr2 = qsFL1( "descr. 2" );

    setValue( parData );

    chartAllData.init();

    chartData1.init();
    chartData2.init();
    chartData3.init();
    chartData4.init();
}

QLineChart::~QLineChart()
{
}

/***************************************************************************
    set values and descriptions text
 ***************************************************************************/

void QLineChart::setValue( struct T_ParData &parData )
{
    struct T_ValData valData;

    valData = valChart;

    if ( parData.enableValue1 )
        valData.qsValue1 = parData.qvValue1.toString();
    if ( parData.enableValue2 )
        valData.qsValue2 = parData.qvValue2.toString();
    if ( parData.enableValue3 )
        valData.qsValue3 = parData.qvValue3.toString();
    if ( parData.enableValue4 )
        valData.qsValue4 = parData.qvValue4.toString();
    if ( parData.enableDescr1 )
        valData.qsDescr1 = parData.qvDescr1.toString();
    if ( parData.enableDescr2 )
        valData.qsDescr2 = parData.qvDescr2.toString();

    if ( parData.enableValue1 && valData.qsValue1 != valChart.qsValue1 )
        qlValue1->setText( valData.qsValue1 );
    if ( parData.enableValue2 && valData.qsValue2 != valChart.qsValue2 )
        qlValue2->setText( valData.qsValue2 );
    if ( parData.enableValue3 && valData.qsValue3 != valChart.qsValue3 )
        qlValue3->setText( valData.qsValue3 );
    if ( parData.enableValue4 && valData.qsValue4 != valChart.qsValue4 )
        qlValue4->setText( valData.qsValue4 );
    if ( parData.enableDescr1 && valData.qsDescr1 != valChart.qsDescr1 )
        qlDescr1->setText( valData.qsDescr1 );
    if ( parData.enableDescr2 && valData.qsDescr2 != valChart.qsDescr2 )
        qlDescr2->setText( valData.qsDescr2 );

    valChart = valData;
} // void setValue( struct T_ParData &parData )

/***************************************************************************
    set values and descriptions visibility
 ***************************************************************************/

void QLineChart::setValueVisible( struct T_ParData &parData )
{
    struct T_ValData valData;

    valData = valChart;

    if ( parData.enableValue1 )
        valData.visibleValue1 = parData.qvValue1.toBool();
    if ( parData.enableValue2 )
        valData.visibleValue2 = parData.qvValue2.toBool();
    if ( parData.enableValue3 )
        valData.visibleValue3 = parData.qvValue3.toBool();
    if ( parData.enableValue4 )
        valData.visibleValue4 = parData.qvValue4.toBool();
    if ( parData.enableDescr1 )
        valData.visibleDescr1 = parData.qvDescr1.toBool();
    if ( parData.enableDescr2 )
        valData.visibleDescr2 = parData.qvDescr2.toBool();

    if ( valData.visibleValue1 != valChart.visibleValue1 )
        qlValue1->setVisible( valData.visibleValue1 );
    if ( valData.visibleValue2 != valChart.visibleValue2 )
        qlValue2->setVisible( valData.visibleValue2 );
    if ( valData.visibleValue3 != valChart.visibleValue3 )
        qlValue3->setVisible( valData.visibleValue3 );
    if ( valData.visibleValue4 != valChart.visibleValue4 )
        qlValue4->setVisible( valData.visibleValue4 );
    if ( valData.visibleDescr1 != valChart.visibleDescr1 )
        qlDescr1->setVisible( valData.visibleDescr1 );
    if ( valData.visibleDescr2 != valChart.visibleDescr2 )
        qlDescr2->setVisible( valData.visibleDescr2 );

    valChart = valData;
} // void setValueVisible( struct T_ParData &parData )

/***************************************************************************
    set the values and descriptions color
 ***************************************************************************/

void QLineChart::setValueColor( struct T_ParData &parData )
{
    if ( parData.enableValue1 )
    {
        valChart.qcValue1 = parData.qvValue1.value<QColor>();

        qlValue1->setPalette( getPaletteColor( valChart.qcValue1 ) );

        if ( !valChart.visibleValue4 && qlsSeries1 )
        {
            qpSeries.setColor( valChart.qcValue1 );
            qlsSeries1->setPen( qpSeries );
        }
    }
    if ( parData.enableValue2 )
    {
        valChart.qcValue2 = parData.qvValue2.value<QColor>();

        qlValue2->setPalette( getPaletteColor( valChart.qcValue2 ) );

        if ( qlsSeries2 )
        {
            qpSeries.setColor( valChart.qcValue2 );
            qlsSeries2->setPen( qpSeries );
        }
    }
    if ( parData.enableValue3 )
    {
        valChart.qcValue3 = parData.qvValue3.value<QColor>();

        qlValue3->setPalette( getPaletteColor( valChart.qcValue3 ) );

        if ( qlsSeries3 )
        {
            qpSeries.setColor( valChart.qcValue3 );
            qlsSeries3->setPen( qpSeries );
        }
    }
    if ( parData.enableValue4 )
    {
        valChart.qcValue4 = parData.qvValue4.value<QColor>();

        qlValue4->setPalette( getPaletteColor( valChart.qcValue4 ) );

        if ( qlsSeries4 )
        {
            qpSeries.setColor( valChart.qcValue4 );
            qlsSeries4->setPen( qpSeries );
        }
    }
    if ( parData.enableDescr1 )
    {
        valChart.qcDescr1 = parData.qvDescr1.value<QColor>();

        qlDescr1->setPalette( getPaletteColor( valChart.qcDescr1 ) );
    }
    if ( parData.enableDescr2 )
    {
        valChart.qcDescr2 = parData.qvDescr2.value<QColor>();

        qlDescr2->setPalette( getPaletteColor( valChart.qcDescr2 ) );
    }

} // void setValueColor( struct T_ParData &parData )

/***************************************************************************
    return the palette color
 ***************************************************************************/

QPalette QLineChart::getPaletteColor( QColor qcColor )
{
    QPalette qpPalette;

    qpPalette.setColor( QPalette::Disabled, QPalette::WindowText, qcColor );

    return qpPalette;
}

/***************************************************************************
    add chart value
 ***************************************************************************/

void QLineChart::addValue( struct T_ParData &parData, qreal qrXAxisStepIn )
{
    chartAllData.qrMaxValue = -qrealMAX;
    chartAllData.qrMinValue = qrealMAX;

    if ( parData.enableValue1 )
    {
        if ( !qlsSeries1 )
        {
            qlsSeries1 = new QLineSeries();
            qcChart.addSeries( qlsSeries1 );
            qlsSeries1->setName( qsFL1 ( "Line 1" ) );
            qpSeries.setColor( valChart.qcValue1 );
            qlsSeries1->setPen( qpSeries );
            qlsSeries1->attachAxis( &qvaXAxis );
            qlsSeries1->attachAxis( &qvaYAxis );
        }

        addValue ( chartData1, parData.qvValue1.toString() );
    }
    else if ( qlsSeries1 )
    {
        qcChart.removeSeries( qlsSeries1 );
        qlsSeries1 = nullptr;
    }

    if ( parData.enableValue2 )
    {
        if ( !qlsSeries2 )
        {
            qlsSeries2 = new QLineSeries();
            qcChart.addSeries( qlsSeries2 );
            qlsSeries2->setName( qsFL1 ( "Line 2" ) );
            qpSeries.setColor( valChart.qcValue2 );
            qlsSeries2->setPen( qpSeries );
            qlsSeries2->attachAxis( &qvaXAxis );
            qlsSeries2->attachAxis( &qvaYAxis );
        }

        addValue ( chartData2, parData.qvValue2.toString() );
    }
    else if ( qlsSeries2 )
    {
        qcChart.removeSeries( qlsSeries2 );
        qlsSeries2 = nullptr;
    }

    if ( parData.enableValue3 )
    {
        if ( !qlsSeries3 )
        {
            qlsSeries3 = new QLineSeries();
            qcChart.addSeries( qlsSeries3 );
            qlsSeries3->setName( qsFL1 ( "Line 3" ) );
            qpSeries.setColor( valChart.qcValue3 );
            qlsSeries3->setPen( qpSeries );
            qlsSeries3->attachAxis( &qvaXAxis );
            qlsSeries3->attachAxis( &qvaYAxis );
        }

        addValue ( chartData3, parData.qvValue3.toString() );
    }
    else if ( qlsSeries3 )
    {
        qcChart.removeSeries( qlsSeries3 );
        qlsSeries3 = nullptr;
    }

    if ( parData.enableValue4 )
    {
        if ( !qlsSeries4 )
        {
            qlsSeries4 = new QLineSeries();
            qcChart.addSeries( qlsSeries4 );
            qlsSeries4->setName( qsFL1 ( "Line 4" ) );
            qpSeries.setColor( valChart.qcValue4 );
            qlsSeries4->setPen( qpSeries );
            qlsSeries4->attachAxis( &qvaXAxis );
            qlsSeries4->attachAxis( &qvaYAxis );
        }

        addValue ( chartData4, parData.qvValue4.toString() );
    }
    else if ( qlsSeries4 )
    {
        qcChart.removeSeries( qlsSeries4 );
        qlsSeries4 = nullptr;
    }


    setMinMaxYAxis ( chartAllData, &qvaYAxis );

    if ( qlsSeries1 )
        updateSeries( chartData1, qlsSeries1 );
    if ( qlsSeries2 )
        updateSeries( chartData2, qlsSeries2 );
    if ( qlsSeries3 )
        updateSeries( chartData3, qlsSeries3 );
    if ( qlsSeries4 )
        updateSeries( chartData4, qlsSeries4 );


    if ( qrealCompare ( qrXAxisStepIn, qrXAxisStep ) )
    {
        qrXAxisStep = qrXAxisStepIn;

        update();
    }

} // void QLineChart::addValue( struct T_ValData &valData, qreal qrXAxisStepIn )

void QLineChart::addValue( struct T_ChartData &chartData, QString qsYAxisValueIn )
{
    bool enabOver, enabAdd, enabFirst;

    qreal minXAxis, maxXAxis;
    
    QPointF qpfPoint;

    QList<QPointF>::reverse_iterator riQlPoint, riQlSuccPoint;
    
    minXAxis = qvaXAxis.min();
    maxXAxis = qvaXAxis.max(); 

    chartData.qrMaxValue = -qrealMAX;
    chartData.qrMinValue = qrealMAX;

    enabOver = enabAdd = false;
    enabFirst = true;

    riQlSuccPoint = chartData.dataList.rend();

    for ( riQlPoint = chartData.dataList.rbegin(); !enabOver || riQlPoint != chartData.dataList.rend(); !enabOver ? riQlPoint++ : riQlPoint )
    {
        enabOver = riQlPoint == chartData.dataList.rend();

        if ( enabFirst )
        {
            if ( riQlPoint != chartData.dataList.rend() )
                qpfPoint = *riQlPoint;

            if ( qrealCompare( qpfPoint.x(), maxXAxis  ) < 0 )
                enabAdd = true;

            enabFirst = false;
        }

        if ( !enabOver )
        {
            qpfPoint = *riQlPoint;

            qpfPoint.setX ( qpfPoint.x() + qrXAxisStep );

            if ( enabAdd )
            {
                *riQlPoint = qpfPoint;

                setMinMaxValue ( chartAllData, chartData, &qpfPoint );
            }
            else
            {
                if ( riQlSuccPoint != chartData.dataList.rend() )
                {
                    *riQlSuccPoint = qpfPoint;

                    setMinMaxValue ( chartAllData, chartData, &qpfPoint );
                }

                riQlSuccPoint = riQlPoint;
            }
        }
    }

    if ( enabAdd )
        chartData.dataList.prepend( qpfPoint );

    if ( !chartData.dataList.isEmpty() )
    {
        qpfPoint.setX ( minXAxis );
        qpfPoint.setY ( qsYAxisValueIn.toDouble() );
        chartData.dataList.first() = qpfPoint;

        setMinMaxValue ( chartAllData, chartData, &qpfPoint );
    }

} // void addValue ( QLineSeries *qlsSeries, QString qsYAxisValueIn, qreal qrXAxisStepIn )

/***************************************************************************
    set the y-axis range
 ***************************************************************************/

void QLineChart::setMinMaxValue( struct T_ChartData &chartAllData, struct T_ChartData &chartData, QPointF *qpfValue )
{ 
    if ( qpfValue )
    {
        if ( qrealCompare( qpfValue->y(), chartData.qrMaxValue  ) > 0 )
            chartData.qrMaxValue = qpfValue->y();
        if ( qrealCompare( qpfValue->y(), chartData.qrMinValue  ) < 0 )
            chartData.qrMinValue = qpfValue->y();
        
        if ( qrealCompare( qpfValue->y(), chartAllData.qrMaxValue  ) > 0 )
            chartAllData.qrMaxValue = qpfValue->y();
        if ( qrealCompare( qpfValue->y(), chartAllData.qrMinValue  ) < 0 )
            chartAllData.qrMinValue = qpfValue->y();
    }
}

void QLineChart::setMinMaxYAxis( struct T_ChartData &chartData, QValueAxis *qvaValue )
{
    qreal qrAxisMax, qrAxisMin, qrAxisMaxNew, qrAxisMinNew;
    qreal qrAutoStep, qrAxisRange, qrRifMax, qrRifMin;

    qrAutoStep = 0.05;

    if ( qvaValue )
    {
        if ( qrealCompare( chartData.qrMaxValue, chartData.qrMinValue ) == 0 )
        {
            chartData.qrMaxValue += abs ( chartData.qrMaxValue );
            chartData.qrMinValue -= abs ( chartData.qrMinValue );
        }
        qrAxisMaxNew = qrAxisMax = qvaValue->max();
        qrAxisMinNew = qrAxisMin = qvaValue->min();
        qrAxisRange = qrAxisMax - qrAxisMin;
        qrRifMax = chartData.qrMaxValue + abs ( ( qrAxisRange * qrAutoStep ) );
        qrRifMin = chartData.qrMinValue - abs ( ( qrAxisRange * qrAutoStep ) );

//        qlValue1->setText( qsFL1( "VR %1 %2 \n Mm %3 %4 \n RF %5 %6" ).arg( qlValue1->text() ).arg( qrAxisRange ).arg( chartData.qrMaxValue ).arg( chartData.qrMinValue ).arg( qrRifMax ).arg( qrRifMin ) );

        if ( qrealCompare( chartData.qrMaxValue, qrAxisMax ) > 0)
            qrAxisMaxNew = chartData.qrMaxValue;
        else if ( qrealCompare( qrRifMax, qrAxisMax ) <= 0 )
            qrAxisMaxNew = chartData.qrMaxValue;

        if ( qrealCompare( chartData.qrMinValue, qrAxisMin ) < 0 )
            qrAxisMinNew = chartData.qrMinValue;
        else if ( qrealCompare( qrRifMin, qrAxisMin ) >= 0 )
            qrAxisMinNew = chartData.qrMinValue;


        if ( qrealCompare ( qrAxisMaxNew, qrAxisMax ) != 0 )
            qvaValue->setMax( roundBoundValue ( qrAxisMaxNew, true ) );

        if ( qrealCompare ( qrAxisMinNew, qrAxisMin ) != 0 )
            qvaValue->setMin( roundBoundValue ( qrAxisMinNew, false ) );


        chartData.qrOldMaxValue = chartData.qrMaxValue;
        chartData.qrOldMinValue = chartData.qrMinValue;

    } // if ( qvaValue )
} // void setMinMaxYAxis( struct T_ChartData &chartData, QValueAxis *qvaValue )

void QLineChart::setMinMaxYAxis_old( struct T_ChartData &chartData, QValueAxis *qvaValue )
{
    qreal qrAxisMax, qrAxisMin, qrAxisMaxRif, qrAxisMinRif, qrAxisMaxNew, qrAxisMinNew;
    qreal qrRifMaxMax, qrRifMaxMin, qrRifMinMax, qrRifMinMin;
    qreal qrAxisRange, qrMaxStep, qrMinStep, qrAutoStep;

    qrAutoStep = 0.05;

    
    if ( qvaValue )
    {
        qrAxisMaxNew = qrAxisMaxRif = qrAxisMax = qvaValue->max();
        qrAxisMinNew = qrAxisMinRif = qrAxisMin = qvaValue->min();
        qrAxisRange = qrAxisMax - qrAxisMin;

//      qrRifMaxMin = chartData.qrMaxValue + abs ( floor ( ( chartData.qrMaxValue * qrAutoStep ) ) );
        qrRifMaxMin = chartData.qrMaxValue + abs ( floor ( ( qrAxisRange * qrAutoStep ) ) );
//      qrRifMaxMax = chartData.qrMaxValue + abs ( ceil ( chartData.qrMaxValue * qrAutoStep ) );
        qrRifMaxMax = chartData.qrMaxValue + abs ( ceil ( qrAxisRange * qrAutoStep ) );

//      qrRifMinMin = chartData.qrMinValue - abs ( ceil ( ( chartData.qrMinValue * qrAutoStep ) ) );
        qrRifMinMin = chartData.qrMinValue - abs ( ceil ( ( qrAxisRange * qrAutoStep ) ) );
//      qrRifMinMax = chartData.qrMinValue - abs ( floor ( chartData.qrMinValue * qrAutoStep ) );
        qrRifMinMax = chartData.qrMinValue - abs ( floor ( qrAxisRange * qrAutoStep ) );
        
        if ( qrealCompare( chartData.qrMaxValue, chartData.qrOldMaxValue ) < 0 )
        {
            if ( qrealCompare( qrAxisMaxRif, qrRifMaxMin ) > 0 )
            {
                qrMaxStep = qrealDivision ( ( 2.0 * qrAutoStep ),  (1 - ( 2.0 * qrAutoStep ) ) );
//              qrAxisMaxNew = ceil ( chartData.qrMaxValue + abs ( ceil ( chartData.qrMaxValue *  qrMaxStep ) ) );
                qrAxisMaxNew = ceil ( chartData.qrMaxValue + abs ( ceil ( qrAxisRange *  qrMaxStep ) ) );

                if ( qrealCompare( qrAxisMaxNew, qrAxisMaxRif ) < 0 )
                    qrAxisMaxRif = qrAxisMaxNew;
                else
                    qrAxisMaxNew = qrAxisMaxRif;
            }
        } // if ( qrealCompare( charData.qrMaxValue, charData.qrOldMaxValue ) < 0 )
        else
        {
            if ( qrealCompare( qrAxisMaxRif, qrRifMaxMax ) < 0 )
            {
                qrMaxStep = qrealDivision ( ( 2.0 * qrAutoStep ),  (1 - ( 2.0 * qrAutoStep ) ) );
//              qrAxisMaxNew = ceil ( chartData.qrMaxValue + abs ( ceil ( chartData.qrMaxValue * qrMaxStep ) ) );
                qrAxisMaxNew = ceil ( chartData.qrMaxValue + abs ( ceil ( qrAxisRange * qrMaxStep ) ) );

                if ( qrealCompare( qrAxisMaxNew, qrAxisMaxRif ) > 0 )
                    qrAxisMaxRif = qrAxisMaxNew;
                else
                    qrAxisMaxNew = qrAxisMaxRif;
            }

            if ( qrealCompare ( qrAxisMaxNew, qrAxisMax ) != 0 )
                qvaValue->setMax( roundBoundValue ( qrAxisMaxNew, true ) );
        } // else ( qrealCompare( charData.qrMaxValue, charData.qrOldMaxValue ) < 0 )


        qrAxisMaxNew = qrAxisMaxRif;
        if ( qrealCompare ( qrAxisMaxNew, qrAxisMax ) != 0 )
            qvaValue->setMax( roundBoundValue ( qrAxisMaxNew, true ) );
        


        if ( qrealCompare( chartData.qrMinValue, chartData.qrOldMinValue ) > 0 )
        {
            if ( qrealCompare( qrAxisMinRif, qrRifMinMin ) < 0 )
            {
                qrMinStep = qrealDivision ( ( 2.0 * qrAutoStep ),  (1 - ( 2.0 * qrAutoStep ) ) );
//              qrAxisMinNew = floor ( chartData.qrMinValue - abs (floor ( chartData.qrMinValue *  qrMinStep ) ) );
                qrAxisMinNew = floor ( chartData.qrMinValue - abs (floor ( qrAxisRange *  qrMinStep ) ) );

                if ( qrealCompare( qrAxisMinNew, qrAxisMinRif ) > 0 )
                    qrAxisMinRif = qrAxisMinNew;
                else
                    qrAxisMinNew = qrAxisMinRif;
            }
        } // if ( qrealCompare( charData.qrMinValue, charData.qrOldMinValue ) > 0 )
        else
        {
            if ( qrealCompare( qrAxisMinRif, qrRifMinMax ) > 0 )
            {
                qrMinStep = qrealDivision ( ( 2.0 * qrAutoStep ),  (1 - ( 2.0 * qrAutoStep ) ) );
//              qrAxisMinNew = floor ( chartData.qrMinValue - abs ( floor ( chartData.qrMinValue * qrMinStep ) ) );
                qrAxisMinNew = floor ( chartData.qrMinValue - abs ( floor ( qrAxisRange * qrMinStep ) ) );

                if ( qrealCompare( qrAxisMinNew, qrAxisMinRif ) < 0 )
                    qrAxisMinRif = qrAxisMinNew;
                else
                    qrAxisMinNew = qrAxisMinRif;
            }

            if ( qrealCompare ( qrAxisMinNew, qrAxisMin ) != 0 )
                qvaValue->setMin( roundBoundValue ( qrAxisMinNew, false ) );
        } // else ( qrealCompare( charData.qrMinValue, charData.qrOldMinValue ) > 0 )


        qrAxisMinNew = qrAxisMinRif;
        if ( qrealCompare ( qrAxisMinNew, qrAxisMin ) != 0 )
            qvaValue->setMin( roundBoundValue ( qrAxisMinNew, false ) );



        chartData.qrOldMaxValue = chartData.qrMaxValue;
        chartData.qrOldMinValue = chartData.qrMinValue;

    } // if ( qvaValue )
} // void setMinMaxYAxis( struct T_ChartData &chartData, QValueAxis *qvaValue )

/***************************************************************************
    update chart series
 ***************************************************************************/

void QLineChart::updateSeries( struct T_ChartData &charData, QAbstractSeries *series )
{    
    QXYSeries *qxySeries;
    
    if ( series )
    {
        qxySeries = static_cast<QXYSeries *>( series );

        qxySeries->replace( charData.dataList );
    }
}

/***************************************************************************
    update chart
 ***************************************************************************/

void QLineChart::update()
{
    int sizeFontWidth, sizeFontHeight, sizeFontChart;
    int sizeFontYAxis, sizeFontXAxis, sizeFontValue1;

    qreal sizePercWidth, sizePercHeight;
    qreal maxAxis;
    qreal percPos;

    QSize qsLChart;
    QRect qrValue, qrDescr;
    QFont qfYAxis, qfXAxis, qfValue, qfDescr;

    qsLChart = size();


//    QRectF qrfPlotArea = qcChart.plotArea(); 
//    qDebug() << "\033[91m" << "qDebug(): " << qrfPlotArea << qsLChart.width() << qsLChart.height() <<"\033[0m";


    sizePercWidth = qsLChart.width() / 480.0;
    sizePercHeight = qsLChart.height() / 480.0;

    sizeFontValue1 = 7;
    sizeFontWidth = 5 + ( sizePercWidth * 24 );
    sizeFontHeight = 5 + ( sizePercHeight * 24 );
    sizeFontChart = ( sizeFontWidth < sizeFontHeight ) ? sizeFontWidth : sizeFontHeight;
    if ( sizeFontChart > sizeFontValue1 )
        sizeFontValue1 = sizeFontChart;

    sizeFontYAxis = 7;
    if ( sizeFontValue1 >= 14 )
        sizeFontYAxis = 10;
    else if ( sizeFontValue1 >= 11 )
        sizeFontYAxis = sizeFontValue1 - 4;
    sizeFontXAxis = sizeFontYAxis;


//  qfDescr.setFamily();
    qfDescr.setPointSize( ( sizeFontValue1 >= 11 ) ? ( sizeFontValue1 - 1 ) : 10 );    
    qfDescr.setBold( false );
    qfDescr.setWeight( QFont::Light );
    qfDescr.setStretch( QFont::Condensed );


    tickYAxis = 5;
    tickXAxis = 5;

    maxAxis = 150.0 * qrXAxisStep  * sizePercWidth;
    maxAxis = qRound ( maxAxis / 10.0 ) * 10.0;

    if ( qrealCompare( sizePercHeight, 0.35 ) < 0 )
       tickYAxis = 4;

    if ( qrealCompare( sizePercHeight, 0.25 ) < 0 )
       tickYAxis = 2;

    if ( qrealCompare( sizePercWidth, 0.55 ) >= 0 && qFuzzyIsNull ( remainder( maxAxis, 10.0 ) ) )
        tickXAxis = 10;

    if ( qrealCompare( sizePercWidth, 0.25 ) < 0 )
    {
        tickXAxis = 2;
        
        qfDescr.setPointSize( 7 );
        qfDescr.setStretch( QFont::UltraCondensed );
    }

    qfYAxis.setPointSize( sizeFontYAxis );
    qfYAxis.setBold( false );

    qfXAxis = qfYAxis;
    qfXAxis.setPointSize( sizeFontXAxis );

//  qfValue1.setFamily();
    qfValue.setPointSize( sizeFontValue1 );
    qfValue.setBold( false );

    qvaYAxis.setTickCount( tickYAxis + 1 );
    qvaYAxis.setLabelsFont( qfYAxis );

    qvaXAxis.setTickCount( tickXAxis + 1 );
    qvaXAxis.setRange( 0, maxAxis);
    qvaXAxis.setLabelsFont( qfXAxis );


    percPos = 2.0;
    if ( valChart.visibleValue4 )
        percPos = 10.0;

    qlValue1->setFont( qfValue );
    qlValue1->adjustSize();
    qlValue1->move( 0, ( ( qsLChart.height() - qlValue1->height() ) / percPos ) );
    qlValue1->setFixedWidth( qsLChart.width() );
    qlValue1->setAlignment( Qt::AlignHCenter );

    if ( valChart.visibleValue4 )
    {        
        qrValue = qlValue1->geometry();

        qlValue2->setFont( qfValue );
        qlValue2->adjustSize();
        qlValue2->move( 0, ( qrValue.y() + qrValue.height() ) );
        qlValue2->setFixedWidth( qsLChart.width() );
        qlValue2->setAlignment( Qt::AlignHCenter );

        qrValue = qlValue2->geometry();

        qlValue3->setFont( qfValue );
        qlValue3->adjustSize();
        qlValue3->move( 0, ( qrValue.y() + qrValue.height() ) );
        qlValue3->setFixedWidth( qsLChart.width() );
        qlValue3->setAlignment( Qt::AlignHCenter );

        qrValue = qlValue3->geometry();

        qlValue4->setFont( qfValue );
        qlValue4->adjustSize();
        qlValue4->move( 0, ( qrValue.y() + qrValue.height() ) );
        qlValue4->setFixedWidth( qsLChart.width() );
        qlValue4->setAlignment( Qt::AlignHCenter );
    }

    qlDescr1->setFont( qfDescr );
    qlDescr1->adjustSize();
    qlDescr1->move( 3, ( ( qsLChart.height() - qlDescr1->height() ) / 10.0 ) );
    qlDescr1->setFixedWidth( qsLChart.width() / 2.0 );
    qlDescr1->setAlignment( Qt::AlignHCenter );

    qrDescr = qlDescr1->geometry();

    qlDescr2->setFont( qfDescr );
    qlDescr2->adjustSize();
    qlDescr2->move( ( qrDescr.x() + qrDescr.width() ), ( ( qsLChart.height() - qlDescr2->height() ) / 10.0 ) );
    qlDescr2->setFixedWidth( qsLChart.width() / 2.0 );
    qlDescr2->setAlignment( Qt::AlignHCenter );
}

/***************************************************************************
    widget resize event
 ***************************************************************************/

void QLineChart::resizeEvent ( QResizeEvent *qreEvent )
{
    Q_UNUSED(qreEvent);

    qcvView->setGeometry( 0, 0, width(), height() );
//    qcChart.setGeometry( -10, 0, width(), height() );

    update();
}


/***************************************************************************
    test chart
 ***************************************************************************/

testChart::testChart() : QWidget()
{
    struct QLineChart::T_ParData parData;

    setWindowTitle ( qsFL1 ( "Chart (only for test)" ) );

    QPalette palette3, palette4;
    QBrush brush3, brush4;

    brush3 = QBrush (QColor(255, 255, 255, 255));
    brush3.setStyle(Qt::SolidPattern);

    palette3.setBrush(QPalette::Active, QPalette::Window, brush3);
    palette3.setBrush(QPalette::Inactive, QPalette::Window, brush3);
    palette3.setBrush(QPalette::Disabled, QPalette::Base, brush3);
    palette3.setBrush(QPalette::Disabled, QPalette::Window, brush3);

    brush4 = brush3;
    brush4.setColor(QColor(200, 57, 241, 255));

    palette4 = palette3;
    palette4.setBrush(QPalette::Active, QPalette::Window, brush4);
    palette4.setBrush(QPalette::Inactive, QPalette::Window, brush4);
    palette4.setBrush(QPalette::Disabled, QPalette::Base, brush4);
    palette4.setBrush(QPalette::Disabled, QPalette::Window, brush4);

    qwChart = new QWidget(this);
    qwChart->setObjectName("qwChart");
    qwChart->setGeometry(QRect(0, 0, 96, 96));

    qfChart = new QFrame(qwChart);
    qfChart->setObjectName("qfChart");
//        qfChart->setGeometry(QRect(0, 0, 250, 250));
    qfChart->resize(qwChart->size());

    qfChart->setPalette(palette3);
    qfChart->setAutoFillBackground(true);
    qfChart->setFrameShape(QFrame::Shape::Panel);
    qfChart->setFrameShadow(QFrame::Shadow::Raised);

    qwChart2 = new QWidget(this);
    qwChart2->setObjectName("qwChart2");
    qwChart2->setGeometry(QRect(300, 0, 250, 250));

    qfChart2 = new QFrame(qwChart2);
    qfChart2->setObjectName("qfChart2");
//        qfChart2->setGeometry(QRect(0, 0, 250, 250));
    qfChart2->resize(qwChart2->size());

//        qfChart2->setPalette(palette4);
    qfChart2->setAutoFillBackground(true);
    qfChart2->setFrameShape(QFrame::Shape::Panel);
    qfChart2->setFrameShadow(QFrame::Shadow::Raised);


    qclChart = new QLineChart (qfChart);
    qclChart->setGeometry( 0, 0, 96, 96 );


//    QRectF qrfPlotArea = qclChart->qcChart.plotArea(); 
//    qDebug() << "\033[91m" << "qDebug(): " << qrfPlotArea << size().width() << size().height() <<"\033[0m";


    qclChart2 = new QLineChart (qfChart2);
    qclChart2->setGeometry( 0, 0, 250, 250 );


//    show();

    resize(600, 600);

    testStepY = 0.1;
    testStepX = 0.1;
    testPosY = 0;


    parData.init( true );

    parData.qvValue1 = parData.qvDescr1 = parData.qvDescr2 = true;

    qclChart2->setValueVisible( parData );

/*  parData.qvValue1 = qsFL1 ( "val. 1" );
    parData.qvValue2 = qsFL1 ( "val. 2" );
    parData.qvValue3 = qsFL1 ( "val. 3" );
    parData.qvValue4 = qsFL1 ( "val. 4" );
    parData.qvDescr1 = qsFL1 ( "descr. 1" );
    parData.qvDescr2 = qsFL1 ( "descr. 2" );

    qclChart2->setValue( parData ); */

    parData.init( false );

    parData.enableValue1 = true;
    parData.qvValue1= QString::number ( testPosY );

    qclChart2->setValue( parData );
//  qclChart->addData( testPosY );
    qclChart2->addValue( parData, testStepX );


    idTimer = startTimer( 100 );

//   setCentralWidget(qwChart3);

/*   QMainWindow * win = (QMainWindow *) QApplication::activeWindow();        

     foreach (QWidget *w, qApp->topLevelWidgets())
        if (win = qobject_cast<QMainWindow*>(w))
          break; */
}

testChart::~testChart()
{

}

void testChart::timerEvent( QTimerEvent * )
{
    struct QLineChart::T_ParData parData;

    static int enabKill = 0;

    if ( testPosY == 0.0 )
        testPosY = 2.0;

    testPosY += testStepY;

    if ( enabKill >= 0 && enabKill < 10 )
    {
        if ( testPosY > 3 )
        {        
            testPosY = -20.0;

            if ( enabKill == 1 )
                enabKill = 9;

            enabKill++;
        }
    }
    else if ( enabKill >= 10 && enabKill < 12 )
    {
        if ( testPosY > 30.0 )
        {
            testPosY = 9.0;
            enabKill++;
        }
    }
    else if ( enabKill < 15 )
    {
        if ( testPosY > 12.0 )
        {
            testPosY = 9.0;

            enabKill++;
        }
    }


    parData.init( true );

    parData.qvValue1 = true;

    qclChart2->setValueVisible( parData );

    parData.init( false );

    parData.enableValue1 = true;
    parData.qvValue1 = qsFL1( "TCTL %1 C" ).arg ( testPosY, 0, 'f', 0, QChar::fromLatin1( ' ' ) );

//    qclChart2->setValue( parData );


    parData.qvValue1 = QString::number ( testPosY );

    qclChart2->setValue( parData );
//    qclChart->addData( posY );
    qclChart2->addValue( parData, testStepX );
}
