/***************************************************************************
                          sensorDevsList.cpp  -  description
                             -------------------
    begin                : Mon Aug 6 2001
    copyright            : (C) 2001 by
    email                :
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>

#include <unistd.h>
#include <string.h>
#include <sys/stat.h>

#include <qstringlist.h>
#include <qcoreevent.h>
#include <qapplication.h>
#include <qcollator.h>
#include <qdebug.h>

#include <kaboutdata.h>

#include "general.h"
#include "sensordevslist.h"


//****************************************************************************
// Public methods
//****************************************************************************

SensorDevsList::SensorDevsList( KsensorsMain *ksensorsMainIn, QObject *parent, const QString &name ) : QObject( parent /*,name */ )
{
    Q_UNUSED(name);

    ksensorsMain = ksensorsMainIn;
    
    qmmSensorDevsList.clear();
  
 if(initSensors()) createLMSensors();
 
  createI8KSensors();
  createHDSensors();
  createSysSensors();
}

SensorDevsList::~SensorDevsList()
{
 if(count()) sensors_cleanup();
}

bool SensorDevsList::initSensors()
{
  const KAboutData kadAboutData = KAboutData::applicationData();
  
#if SENSORS_API_VERSION < 0x400 /* libsensor 3 code */
 FILE *fp=fopen("/etc/sensors.conf","r");
 if(!fp) {
   qWarning(kadAboutData.displayName().toLatin1() + " error: /etc/sensors.conf not found !");
   return false;
 }
#else
 FILE *fp=NULL;
#endif
 int err= sensors_init(fp);
 if(err) {
   qWarning( ( kadAboutData.displayName() + qsFL1( " error: sensors_init fail, error code %d" ) ).toLatin1().data(), err );
   return false;
 }
#if SENSORS_API_VERSION < 0x400 /* libsensor 3 code */
 fclose(fp);
#endif
 return true;
}

void SensorDevsList::createLMSensors()
{
 const sensors_chip_name *chip_name;
 int err= 0;
#if SENSORS_API_VERSION < 0x400 /* libsensor 3 code */
 while( (chip_name= sensors_get_detected_chips(&err)) )
 {
   if( existSensor(chip_name,"temp")  ||
       existSensor(chip_name,"fan")   )
       {
          (void)new LMSensorsChip( this, chip_name );
       }
 }
#else
 while( (chip_name= sensors_get_detected_chips(NULL, &err)) )
          (void)new LMSensorsChip( this, chip_name );
#endif
}

void SensorDevsList::createHDSensors()
{
 HDSensorsList *disks= new HDSensorsList( this, qsFL1( "Disks" ) );
 if(disks->count()==0) delete disks;
}

void SensorDevsList::createI8KSensors()
{
 if(I8KSensorsList::I8KAvailable())
   (void) new I8KSensorsList( this, qsFL1( "I8KSensors" ) );
}

void SensorDevsList::createSysSensors()
{
  (void) new SYSSensorsList( this, qsFL1( "System" ) );
}

#if SENSORS_API_VERSION < 0x400 /* libsensor 3 code */
int LMSensors::existSensor(const sensors_chip_name *chip_name,const char *sensor_name)
{
int nr1,nr2;
const sensors_feature_data *sensor_data;

 nr1=nr2= 0;
 while( (sensor_data= sensors_get_all_features(*chip_name, &nr1, &nr2)) )
 {
   if( strstr(sensor_data->name,sensor_name) )
     return sensor_data->number;
 }
 return 0;
}
#endif

void SensorDevsList::setMonitorized(bool enable)
{
 const QObjectList *list= getSensorsChips();
 if(list)
 {
    for ( QObjectList::ConstIterator it = list->begin(); it != list->end(); ++it ) 
    {
      SensorDev *sensor = (SensorDev *)(*it);
 
      if ( sensor )
      {
        sensor->setMonitorized(enable);
      }
      else
        break;
    } 
 }
}

/***************************************************************************
    get the sensors list
 ***************************************************************************/

const QObjectList *SensorDevsList::getSensorsChips() 
{ 
    return &children();    
}

Sensor *SensorDevsList::getSensor( const QString &name )
{
 int     index= count();
 Sensor *sensor = nullptr;
 while(--index>=0 && !sensor) {
    sensor= getSensorsChip(index)->getSensor(name);
 }
 return sensor;
}

void SensorDevsList::childEvent( QChildEvent *e )
{  
//     qDebug() << "\033[91m" << "qDebug(): LMSensors::childEvent()" << "\033[0m";
  
 if(e->added()) 
 {      
//   qDebug() << "\033[91m" << "qDebug(): LMSensors::childEvent()" << "e->added()" << "\033[0m";
      
    connect((SensorDev *)e->child(),SIGNAL(valueChanged(Sensor *)) ,this,SIGNAL(valueChanged(Sensor  *)));
 }
}


/*********************************************************************************/
